@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 
-
 """ Reads in crux/percolator tab-delimited results (psms) and returns tmt values"""
 
 import os.path
@@ -8,7 +7,7 @@ import sys
 import re
 import pandas as pd
 import tqdm
-import logging
+import argparse
 
 from pytmt import __version__
 from pytmt.get_spec import Mzml
@@ -16,7 +15,10 @@ from pytmt import tmt_reporters
 from pytmt import quantify_spec
 from pytmt import correct_matrix
 
-def quant(args):
+from pytmt.logger import get_logger
+from pytmt.protein_group import get_canonical_parsimony_groups
+
+def quant(args) -> None:
     """
      reads in Percolator tab-delimited results (PSMS) \\
      and filter each row by protein-uniqueness and by Percolator q value \\
@@ -44,62 +46,28 @@ def quant(args):
         pytmt tests/data/mzml tests/data/percolator/percolator.target.psms.txt -o out
 
     Example values for arguments:
-        mzml_loc = 'tests/data/mzml'
-        id_loc = 'tests/data/percolator'
+        mzml = 'tests/data/mzml'
+        id = 'tests/data/percolator'
         precision = 10
-        q_filter = 0.1
-        unique_only = True
+        qvalue = 0.1
 
     :param args:    arguments from argparse
     :return:        Exit OK
     """
 
-    # Main logger setup
-    main_log = logging.getLogger('pytmt')
-    main_log.setLevel(logging.DEBUG)
-
-    # create file handler which logs even debug messages
-    os.makedirs(args.out, exist_ok=True)
-    fh = logging.FileHandler(os.path.join(args.out, 'tmt.log'))
-    fh.setLevel(logging.DEBUG)
-
-    # create console handler with a higher log level
-    ch = logging.StreamHandler()
-    ch.setLevel(logging.INFO)
-
-    # create formatter and add it to the handlers
-    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
-    fh.setFormatter(formatter)
-    ch.setFormatter(formatter)
-
-    # add the handlers to the logger
-    main_log.addHandler(fh)
-    main_log.addHandler(ch)
-
-    main_log.info(args)
-    main_log.info(__version__)
-
-    # Folders of mzML files and the Percolator results.
-    mzml_loc = args.mzml
-    id_loc = args.id
-
-    assert args.multiplex in [0, 2, 6, 10, 11, 16, 18], '[error] TMT multiplexity not 0, 2, 6, 10, 11, 16, or 18'
+    # ---- Get the logger ----
+    logger = get_logger(__name__, args.out)
+    logger.info(args)
+    logger.info(__version__)
 
     # Get the reporter masses
     reporters = tmt_reporters.get_reporters(args.multiplex)
 
     # Define the PPM of integration
     precision = args.precision
-    assert 1 <= precision <= 1000, '[error] mass tolerance must be between 1 and 1000 ppm'
 
-    # Define the Percolator Q value cutoff filter
-    q_filter = args.qvalue
-
-    # Define the Percolator protein-unique PSM filter
-    unique_only = args.unique
-
-    assert os.path.isdir(mzml_loc), '[error] mzml directory path not valid'
-    assert os.path.isfile(id_loc), '[error] percolator file path not valid'
+    assert os.path.isdir(args.mzml), '[error] mzml directory path not valid'
+    assert os.path.isfile(args.id.name), '[error] percolator file path not valid'
 
     # List all files in the percolator directory ending with target.psms.txt.
         # id_files = [f for f in os.listdir(id_loc) if f.endswith('target.psms.txt')]
@@ -107,16 +75,24 @@ def quant(args):
 
     # Read the Percolator psms file.
     try:
-        id_df = pd.read_csv(filepath_or_buffer=id_loc,  # os.path.join(id_loc, id_files[0]),
+        id_df = pd.read_csv(filepath_or_buffer=args.id,  # os.path.join(id_loc, id_files[0]),
                             sep='\t')
-    except pd.errors.ParserError:
-        main_log.info("Pandas ParserError: trying readlines for possible standalone Percolator file")
 
-        with open(id_loc, 'r') as f:
+    except pd.errors.EmptyDataError:
+        logger.error('Unable to read percolator')
+        sys.exit(1)
+
+    try:
+        id_df['file_idx']
+
+    except KeyError:
+        logger.info("Unable to find file_idx, attempting to read as standalone Percolator file")
+
+        # The standalone percolator output has different number of columns per row because the proteins are separated by tabs
+        # Read the first half of the table without the protein IDs
+        with open(args.id.name, 'r') as f:
             f_ln = f.readlines()
 
-        # The percolator output has different number of columns per row because the proteins are separated by tabs
-        # Read the first half of the table without the protein IDs
         id_df = pd.DataFrame([ln.split('\t')[0:5] for ln in f_ln[1:]])
         id_df.columns = ['PSMId', 'score', 'percolator q-value', 'posterior_error_prob', 'peptide']
         id_df['percolator q-value'] = id_df['percolator q-value'].astype(float)
@@ -147,11 +123,11 @@ def quant(args):
     file_indices = list(set(id_df['file_idx']))
 
     # 2022-03-28 pytmt will now attempt to read the percolator.log.txt file for fraction (file_idx) mzML assignment
-    log_path = os.path.join(os.path.dirname(id_loc), 'percolator.log.txt')
+    log_path = os.path.join(os.path.dirname(args.id.name), 'percolator.log.txt')
 
     # If the log file exists, use it to read the assignment
     if os.path.exists(log_path):
-        main_log.warning(f'Percolator log file exists at {log_path} and will be used for index assignment.')
+        logger.warning(f'Percolator log file exists at {log_path} and will be used for index assignment.')
 
         with open(log_path, 'r') as f:
             lines = f.readlines()
@@ -167,12 +143,12 @@ def quant(args):
 
     # If the log file does not exist, assign index naively based on sort
     else:
-        main_log.warning(f'Percolator log file not found at {log_path}; '
+        logger.warning(f'Percolator log file not found at {log_path}; '
                          f'mzml files will be sorted for index assignment.')
         # Check that the number of mzMLs in the mzML folder is the same as the maximum of the ID file's file_idx column.
         # Note this will throw an error if not every fraction results in at least some ID, but we will ignore for now.
 
-        mzml_filelist = [f for f in os.listdir(mzml_loc) if re.match('^.*.mzML', f)]
+        mzml_filelist = [f for f in os.listdir(args.mzml) if re.match('^.*.mzML', f)]
         # Sort the mzML files by names
         # Note this may create a problem if the OS Percolator runs on has natural sorting (xxxx_2 before xxxx_10)
         # But we will ignore for now
@@ -188,22 +164,21 @@ def quant(args):
         assert len(mzml_files) == max(file_indices) + 1, '[error] number of mzml files not matching id list'
 
     # Print mzml files to log
-    main_log.info(f'mzml file orders: {mzml_files}')
+    logger.info(f'mzml file orders: {mzml_files}')
 
     # For each file index (fraction), open the mzML file, and create a subset Percolator ID dataframe
     for idx in file_indices:
 
         # 2022-03-28 try to open either mzML or mzML.gz
-        # TODO: may have to account for .mzml and mzml.gz rather than .mzML
-        if os.path.exists(os.path.join(mzml_loc, mzml_files[idx] + '.mzML')):
-            mzml_path = os.path.join(mzml_loc, mzml_files[idx] + '.mzML')
-        elif os.path.exists(os.path.join(mzml_loc, mzml_files[idx] + '.mzML.gz')):
-            mzml_path = os.path.join(mzml_loc, mzml_files[idx] + '.mzML.gz')
+        if os.path.exists(os.path.join(args.mzml, mzml_files[idx] + '.mzML')):
+            mzml_path = os.path.join(args.mzml, mzml_files[idx] + '.mzML')
+        elif os.path.exists(os.path.join(args.mzml, mzml_files[idx] + '.mzML.gz')):
+            mzml_path = os.path.join(args.mzml, mzml_files[idx] + '.mzML.gz')
         else:
-            raise FileNotFoundError
+            raise FileNotFoundError(f'Could not find mzML file for index {idx} at {args.mzml}')
 
         # Logging mzML
-        main_log.info(f'Reading mzml file: {os.path.basename(mzml_path)} ({idx + 1} of {len(file_indices)})')
+        logger.info(f'Reading mzml file: {os.path.basename(mzml_path)} ({idx + 1} of {len(file_indices)})')
 
         # Make a subset dataframe with the current file index (fraction) being considered
         fraction_id_df = id_df[id_df['file_idx'] == idx]
@@ -213,7 +188,9 @@ def quant(args):
 
         # Open the mzML file
         fraction_mzml = Mzml(path=mzml_path,
-                             precision=precision)
+                             precision=precision,
+                             logger=logger,
+                             )
         fraction_mzml.parse_mzml_ms2()
 
         # Loop through each qualifying row in sub_df_filtered
@@ -223,11 +200,11 @@ def quant(args):
             scan = fraction_id_df.loc[i, 'scan']
 
             # If q-value filter is on, skip any row that fails the filter.
-            if fraction_id_df.loc[i, 'percolator q-value'] > q_filter:
+            if fraction_id_df.loc[i, 'percolator q-value'] > args.qvalue:
                 continue
 
-            # If the protein-unique PSM filter is on, skip any row that fails the filter.
-            if unique_only and len(fraction_id_df.loc[i, 'protein id'].split(',')) > 1:
+            # If the parsimony setting is set to unique, skip any row that fails the filter.
+            if args.parsimony == 'unique' and len(fraction_id_df.loc[i, 'protein id'].split(',')) > 1:
                 continue
 
             # If this is a qualifying row, get the spectrum in the mzML file by scan number
@@ -236,17 +213,14 @@ def quant(args):
 
                 # Tidy this up
                 if spectrum is None:
-                    main_log.error(f'[error] spectrum index {scan} out of bound or is empty')
+                    logger.error(f'[error] spectrum index {scan} out of bound or is empty')
                     raise KeyError
 
             # TODO: This doesn't give a KeyError currently. Need to catch when the id scans don't match the mzml.
             except KeyError:
-                main_log.error('[error] spectrum index out of bound')
+                logger.error('[error] spectrum index out of bound')
                 continue
 
-
-
-
             #except xml.etree.ElementTree.ParseError:
             #    if args.verbosity == 2:
             #        print('[verbosity 2] XML eTree does not appear to be able to read this spectrum',
@@ -265,7 +239,8 @@ def quant(args):
                                                                spectrum=spectrum,
                                                                precision=precision,
                                                                reporters=reporters,
-                                                               digits=2)
+                                                               digits=2,
+                                                               )
 
             # Grow the results into an output list of lists (creating if does not exist)
             try:
@@ -280,8 +255,6 @@ def quant(args):
     for reporter in reporters:
         output_df_columns.append('m' + str(reporter))
 
-
-
     output_df_columns.append('spectrum_int')
     output_df = pd.DataFrame(output_list, columns=output_df_columns)
 
@@ -295,68 +268,146 @@ def quant(args):
     # Final output, merging the input and output tables
     final_df = pd.merge(id_df, output_df, how='left')
 
-    # Create output directory if it does not exist
-    save_path = os.path.join(args.out, 'tmt_out.txt')
+    # Label light and heavy peptides
+    if args.silac:
+        heavy_mods = ["R\\[10.01\\]", "R\\[239.17\\]", "K\\[8.01\\]", "K\\[237.18\\]", "K\\[466.34\\]"]
+        heavy = "|".join(heavy_mods)
 
-    # Save the file
-    final_df.to_csv(save_path, sep='\t')
+        def add_heavy_tag(string: str) -> str:
+            """ add _H to the end of each uniprot accession """
+            return re.sub("(sp\\|)(.+)(\\|.*$)", "\\1\\2_H\\3", string)
 
-    main_log.info("Run completed successfully.")
-    return sys.exit(os.EX_OK)
+        # Add _H to protein names if the peptide is heavy (contains the heavy tag)
+        final_df['protein id'] = [",".join(map(add_heavy_tag, final_df['protein id'][i].split(',')))
+                                  if bool(re.search(heavy, final_df['sequence'][i]))
+                                  else final_df['protein id'][i]
+                                  for i in range(len(final_df.index))]
+
+    # Save the peptide file
+    final_df.to_csv(os.path.join(args.out, 'tmt_out.txt'), sep='\t')
+
+    # Collapse to protein level
+    if args.contam is not None:
+        protein_column_list = ['protein id', ] + [f'm{reporter}_cor' for reporter in reporters]
+    else:
+        protein_column_list = ['protein id'] + [f'm{reporter}' for reporter in reporters]
+
+    protein_df = final_df[protein_column_list]
+
+    # Sum the reporter intensities for each protein
+    if args.parsimony == 'unique':
+        filtered_protein_df = protein_df[protein_df['protein id'].str.count(',') == 0]
+
+    elif args.parsimony == 'all':
+        filtered_protein_df = protein_df
+
+    elif args.parsimony == 'canonical':
+        filtered_protein_df = get_canonical_parsimony_groups(result_df = protein_df,
+                                                             contam=args.contam,
+                                                             reporters=reporters,
+                                                             )
+
+    # Group by protein and sum the reporter intensities
+    filtered_protein_df = filtered_protein_df.groupby('protein id').sum()
+
+    # Remove any rows that are all zeros
+    filtered_protein_df = filtered_protein_df[filtered_protein_df.sum(axis=1) > 0]
+
+    # Save the protein file
+    filtered_protein_df.to_csv(os.path.join(args.out, 'tmt_protein_out.txt'), sep='\t')
+
+    logger.info("Run completed successfully.")
+
+    return None #sys.exit(os.EX_OK)
 
 
-def main():
+class CheckReadableDir(argparse.Action):
+    """ Class to check if directory is readable. """
+    def __call__(self, parser, namespace, values, option_string=None):
+        prospective_dir = values
+        if not os.path.isdir(prospective_dir):
+            raise argparse.ArgumentTypeError("readable_dir:{0} is not a valid path".format(prospective_dir))
+        if os.access(prospective_dir, os.R_OK):
+            setattr(namespace, self.dest, prospective_dir)
+        else:
+            raise argparse.ArgumentTypeError("readable_dir:{0} is not a readable dir".format(prospective_dir))
+
+
+class CheckQValue(argparse.Action):
+    """ Class to check that q values are between 0 and 1. """
+    def __call__(self, parser, namespace, values, option_string=None):
+        try:
+            values = float(values)
+        except ValueError:
+            raise argparse.ArgumentTypeError("%r for q_value not a floating-point literal" % (values,))
+
+        if values < 0.0 or values > 1.0:
+            raise argparse.ArgumentTypeError("%r for q_value not in range [0.0, 1.0]" % (values,))
+        setattr(namespace, self.dest, values)
+
+
+def main() -> None:
     """
     Entry point
 
     :return:
     """
 
-    import argparse
-
     parser = argparse.ArgumentParser(description='pytmt returns ms2 tmt quantification values'
                                                  'from Percolator output and perform contamination'
-                                                 'correction')
+                                                 'correction',
+                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+                                     epilog='For more information, see GitHub repository at '
+                                            'https://github.com/ed-lau/pytmt',
+                                     )
 
     parser.add_argument('mzml',
-                        help='path to folder containing mzml files',
+                        help='<required> path to folder containing mzml files',
+                        type=str,
+                        action=CheckReadableDir,
                         )
 
     parser.add_argument('id',
-                        help='path to percolator target psms output file',
+                        help='<required> path to percolator target psms output file',
+                        type=argparse.FileType('r'),
                         )
 
-    parser.add_argument('-u',
-                        '--unique',
-                        action='store_true',
-                        help='quantify unique peptides only',
+    parser.add_argument('-P', '--parsimony',
+                        help='rule to collapse peptides into the protein level. '
+                        'options: "all" (default; include all peptides), "unique"'
+                             ' (only unique peptides), "canonical" (collapse'
+                             'into parsimony groups).',
+                        choices=['all', 'unique', 'canonical'],
+                        default='all',
                         )
 
     parser.add_argument('-q', '--qvalue',
                         help='quantify peptides with q value below this threshold [default: 1.0]',
+                        metavar='[0, 1]',
                         type=float,
-                        default=1.0)
+                        default=1.0,
+                        action=CheckQValue,
+                        )
 
     parser.add_argument('-m', '--multiplex',
                         help='TMT-plex (0, 2, 6, 10, 11, 16, 18) [default:10]',
+                        choices=[0, 2, 6, 10, 11, 16, 18],
                         type=int,
                         default=10)
 
     parser.add_argument('-p', '--precision',
                         help='ms2 spectrum mass shift tolerance in ppm [default: 10]',
                         type=int,
+                        choices=[0, 100],
+                        metavar='[0, 100]',
                         default=10)
 
     parser.add_argument('-o', '--out', help='name of the output directory [default: tmt_out]',
                         default='tmt_out')
 
-    parser.add_argument('-v', '--version', action='version',
-                        version='%(prog)s {version}'.format(version=__version__))
-
     parser.add_argument('-c', '--contam',
                         help='Path to contaminant matrix csv file.'
                              ' Leave blank to get tmt output without correction',
-                        metavar='CONTAM',
                         type=argparse.FileType('r'),
                         )
 
@@ -365,6 +416,14 @@ def main():
                         help='uses non-negative least square for contamination correction',
                         )
 
+    parser.add_argument('-S', '--silac',
+                        help='mark peptides with SILAC reporter ions',
+                        action='store_true',
+                        )
+
+    parser.add_argument('-v', '--version', action='version',
+                        version='pyTMT {version}'.format(version=__version__))
+
     parser.set_defaults(func=quant)
 
 
@@ -377,5 +436,14 @@ def main():
     # Parse all the arguments
     args = parser.parse_args()
 
+    # Convert args to a dictionary
+    # args_dict = vars(args)
+
     # Run the function in the argument
     args.func(args)
+
+    return None #sys.exit(os.EX_OK)
+
+
+if __name__ == '__main__':
+    main()
\ No newline at end of file
